@@ -1,5 +1,5 @@
 #!/bin/bash
-# $Id: 7122ea3d3051138f03b846512091d30c87d89b0c $
+# $Id: d29ad48ae7aba45af411153895045badd0e64820 $
 
 determine_KEXT() {
 	local kconfig_file_used="${KERNEL_CONFIG}"
@@ -388,6 +388,7 @@ determine_real_args() {
 	set_config_with_override STRING MODPROBEDIR                           CMD_MODPROBEDIR                           "/etc/modprobe.d"
 
 	set_config_with_override BOOL   SPLASH                                CMD_SPLASH                                "no"
+	set_config_with_override BOOL   PLYMOUTH                              CMD_PLYMOUTH                              "no"
 	set_config_with_override BOOL   CLEAR_CACHEDIR                        CMD_CLEAR_CACHEDIR                        "no"
 	set_config_with_override BOOL   POSTCLEAR                             CMD_POSTCLEAR                             "no"
 	set_config_with_override BOOL   MRPROPER                              CMD_MRPROPER                              "yes"
@@ -444,6 +445,7 @@ determine_real_args() {
 	set_config_with_override BOOL   VIRTIO                                CMD_VIRTIO                                "no"
 	set_config_with_override BOOL   MULTIPATH                             CMD_MULTIPATH                             "no"
 	set_config_with_override BOOL   FIRMWARE                              CMD_FIRMWARE                              "no"
+	set_config_with_override BOOL   ALLFIRMWARE                           CMD_ALLFIRMWARE                           "no"
 	set_config_with_override STRING FIRMWARE_DIR                          CMD_FIRMWARE_DIR                          "/lib/firmware"
 	set_config_with_override STRING FIRMWARE_FILES                        CMD_FIRMWARE_FILES
 	set_config_with_override BOOL   FIRMWARE_INSTALL                      CMD_FIRMWARE_INSTALL                      "no"
@@ -1021,6 +1023,21 @@ determine_real_args() {
 			fi
 		fi
 
+		if isTrue "${PLYMOUTH}" && isTrue "${SPLASH}"
+		then
+			gen_die "--plymouth and --splash are mutually exclusive!"
+		fi
+
+		if isTrue "${PLYMOUTH}" && ! isTrue "${FIRMWARE}"
+		then
+			print_warning 3 "--plymouth potentially requires graphics firmware to function! Please configure your --firmware flags appropriately!"
+		fi
+
+		if isTrue "${PLYMOUTH}" && ! isTrue "${ALLRAMDISKMODULES}"
+		then
+			print_warning 3 "--plymouth potentially requires DRM kernel modules to function! Please configure your --ramdisk-modules flags appropriately!"
+		fi
+
 		if isTrue "${SSH}"
 		then
 			local ssh_authorized_keys_file=$(expand_file "${SSH_AUTHORIZED_KEYS_FILE}")
