@@ -1,5 +1,5 @@
 #!/bin/bash
-# $Id: ab7a7cea08c423b465ffcbdda3909279138f9b97 $
+# $Id: c31e15aab2646e71c467f945182ced082199e3eb $
 
 isTrue() {
 	case "$1" in
@@ -22,6 +22,15 @@ isTrue() {
 	return 1
 }
 
+anyTrue() {
+	local x
+	for x
+	do
+		isTrue "${x}" && return 0
+	done
+	return 1
+}
+
 set_color_vars() {
 	if ! isTrue "${NOCOLOR}"
 	then
@@ -287,6 +296,48 @@ is_psf_file() {
 	echo "${file_is_psf}"
 }
 
+is_kext_supported_by_kmod() {
+	[[ ${#} -ne 1 ]] \
+		&& gen_die "$(get_useful_function_stack "${FUNCNAME}")Invalid usage of ${FUNCNAME}(): Function takes exactly one argument (${#} given)!"
+
+	local requested_feature=${1}
+	requested_feature=${requested_feature##*.}
+	requested_feature=${requested_feature^^}
+
+	local is_supported=no
+
+	if [[ "${requested_feature}" == GZ ]]
+	then
+		requested_feature=ZLIB
+	elif [[ "${requested_feature}" == ZST ]]
+	then
+		requested_feature=ZSTD
+	fi
+
+	case "${requested_feature}" in
+		KO)
+			is_supported=yes
+			;;
+		*)
+			local line
+			while read line; do
+				if [[ ! ${line} =~ ^[\+\-] ]]
+				then
+					continue
+				fi
+
+				if [[ ${line} =~ \+${requested_feature} ]]
+				then
+					is_supported=yes
+					break
+				fi
+			done < <("${KMOD_CMD}" -V 2>/dev/null)
+			;;
+	esac
+
+	echo "${is_supported}"
+}
+
 is_valid_ssh_host_keys_parameter_value() {
 	local parameter_value=${1}
 
@@ -1070,13 +1121,13 @@ get_tar_cmd() {
 	local pcmd
 	if [[ "${archive_file}" == *.tar.bz2 ]]
 	then
-		pcmd=$(which pbzip2 2>/dev/null)
+		pcmd=$(type -P pbzip2 2>/dev/null)
 	elif [[ "${archive_file}" == *.tar.xz ]]
 	then
-		pcmd=$(which pxz 2>/dev/null)
+		pcmd="'xz -T$(makeopts_jobs)'"
 	elif [[ "${archive_file}" == *.tar.gz ]]
 	then
-		pcmd=$(which pigz 2>/dev/null)
+		pcmd=$(type -P pigz 2>/dev/null)
 	fi
 
 	if [ -n "${pcmd}" ]
@@ -1853,20 +1904,21 @@ check_disk_space_requirements() {
 			gen_die "--check-free-disk-space-bootdir value '${CHECK_FREE_DISK_SPACE_BOOTDIR}' is not a valid number!"
 		fi
 
-		available_free_disk_space=$(unset POSIXLY_CORRECT && df -BM "${BOOTDIR}" | awk '$3 ~ /[0-9]+/ { print $4 }')
+		available_free_disk_space="$(($(stat -fc '%f * %S' "${BOOTDIR}")))"
 		if [ -n "${available_free_disk_space}" ]
 		then
 			print_info 2 '' 1 0
 			print_info 2 "Checking for at least ${CHECK_FREE_DISK_SPACE_BOOTDIR} MB free disk space in '${BOOTDIR}' ..."
-			print_info 5 "df reading: ${available_free_disk_space}"
+			print_info 5 "stat(1) reading: ${available_free_disk_space} bytes free"
 
-			available_free_disk_space=${available_free_disk_space%M}
-			if [ ${available_free_disk_space} -lt ${CHECK_FREE_DISK_SPACE_BOOTDIR} ]
+			bytes_in_mb=1048576
+			minimum_free_space_bytes_bootdir="$((${CHECK_FREE_DISK_SPACE_BOOTDIR} * ${bytes_in_mb}))"
+			if [ "${available_free_disk_space}" -lt "${minimum_free_space_bytes_bootdir}" ]
 			then
-				gen_die "${CHECK_FREE_DISK_SPACE_BOOTDIR} MB free disk space is required in '${BOOTDIR}' but only ${available_free_disk_space} MB is available!"
+				gen_die "${CHECK_FREE_DISK_SPACE_BOOTDIR} MB free disk space is required in '${BOOTDIR}' but only ${available_free_disk_space} bytes is available!"
 			fi
 		else
-			print_warning 1 "Invalid df value; Skipping free disk space check for '${BOOTDIR}' ..."
+			print_warning 1 "Invalid stat(1) value; Skipping free disk space check for '${BOOTDIR}' ..."
 		fi
 	fi
 
@@ -1890,20 +1942,21 @@ check_disk_space_requirements() {
 			gen_die "--check-free-disk-space-kerneloutputdir value '${CHECK_FREE_DISK_SPACE_KERNELOUTPUTDIR}' is not a valid number!"
 		fi
 
-		available_free_disk_space=$(unset POSIXLY_CORRECT && df -BM "${KERNEL_OUTPUTDIR}" | awk '$3 ~ /[0-9]+/ { print $4 }')
+		available_free_disk_space="$(($(stat -fc '%f * %S' "${KERNEL_OUTPUTDIR}")))"
 		if [ -n "${available_free_disk_space}" ]
 		then
 			print_info 2 '' 1 0
 			print_info 2 "Checking for at least ${CHECK_FREE_DISK_SPACE_KERNELOUTPUTDIR} MB free disk space in '${KERNEL_OUTPUTDIR}' ..."
-			print_info 5 "df reading: ${available_free_disk_space}"
+			print_info 5 "stat(1) reading: ${available_free_disk_space} bytes free"
 
-			available_free_disk_space=${available_free_disk_space%M}
-			if [ ${available_free_disk_space} -lt ${CHECK_FREE_DISK_SPACE_KERNELOUTPUTDIR} ]
+			bytes_in_mb=1048576
+			minimum_free_space_bytes_kerneloutputdir="$((${CHECK_FREE_DISK_SPACE_KERNELOUTPUTDIR} * ${bytes_in_mb}))"
+			if [ "${available_free_disk_space}" -lt "${minimum_free_space_bytes_kerneloutputdir}" ]
 			then
-				gen_die "${CHECK_FREE_DISK_SPACE_KERNELOUTPUTDIR} MB free disk space is required in '${KERNEL_OUTPUTDIR}' but only ${available_free_disk_space} MB is available!"
+				gen_die "${CHECK_FREE_DISK_SPACE_KERNELOUTPUTDIR} MB free disk space is required in '${KERNEL_OUTPUTDIR}' but only ${available_free_disk_space} bytes is available!"
 			fi
 		else
-			print_warning 1 "Invalid df value; Skipping free disk space check for '${KERNEL_OUTPUTDIR}' ..."
+			print_warning 1 "Invalid stat(1) value; Skipping free disk space check for '${KERNEL_OUTPUTDIR}' ..."
 		fi
 	fi
 }
@@ -1932,6 +1985,29 @@ check_distfiles() {
 	fi
 }
 
+# @FUNCTION: install_exe
+# @USAGE: <file> <destination>
+# @DESCRIPTION:
+# Finds an executable binary file and installs it in cases where there may be
+# similarly named shell wrapper scripts. This happens when GNU libtool creates
+# an executable named 'foo' while slibtool creates '.libs/foo' and 'foo' is a
+# shell script that should not be installed.
+install_exe() {
+	local file="${1##*/}"
+	local dest="${2}"
+
+	local dir
+	[[ "${1%/*}" == "${file}" ]] || dir="${1%/*}/"
+
+	[[ -f "${dir}${file}" ]] || gen_die "File '${dir}${file}' does not exist!"
+
+	# Ensure only the binaries are installed and not a similarly named wrapper script
+	find "${S}/${dir}" -type f -name "${file}" -print0 |
+		xargs -0 file | grep executable | grep ELF | cut -f 1 -d : |
+		xargs -I '{}' cp -a '{}' "${dest}" ||
+		gen_die "Failed to copy '${S}/${dir}${file}' to '${dest}'!"
+}
+
 # @FUNCTION: expand_file
 # @USAGE: <file>
 # @DESCRIPTION:
@@ -1952,7 +2028,7 @@ expand_file() {
 	local file="${1}"
 	local expanded_file=
 
-	expanded_file=$(python -c "import os; print(os.path.expanduser('${file}'))" 2>/dev/null)
+	expanded_file=$("${GK_SHARE}/path_expander.py" "${file}" 2>/dev/null)
 	if [ -z "${expanded_file}" ]
 	then
 		# if Python failed for some reason, just reset
@@ -2123,6 +2199,36 @@ make_bootdir_writable() {
 	fi
 }
 
+get_kext_kmod_use_flag() {
+	[[ ${#} -ne 1 ]] \
+		&& gen_die "$(get_useful_function_stack "${FUNCNAME}")Invalid usage of ${FUNCNAME}(): Function takes exactly one argument (${#} given)!"
+
+	local kext=${1}
+	kext=${kext##*.}
+	kext=${kext^^}
+
+	local use_flag=
+
+	case "${kext}" in
+		GZ)
+			use_flag="zlib"
+			;;
+		KO)
+			;;
+		XZ)
+			use_flag="lzma"
+			;;
+		ZST)
+			use_flag="zstd"
+			;;
+		*)
+			gen_die "$(get_useful_function_stack)Internal error: KEXT '${kext}' is unknown!"
+			;;
+	esac
+
+	echo "${use_flag}"
+}
+
 # @FUNCTION: get_nproc
 # @USAGE: [${fallback:-1}]
 # @DESCRIPTION:
